//Projet de statistiques - IUP 1 MIAGE
//Etudes de files d'attente simules par loi exponentielle.
//Compiler avec DJGPP v2.0 et Allegro v3.0
//DJGPP est un compilateur MS-DOS freeware 32 bits deriv du GNU C/C++

//Equipe de programmation:
//------------------------
//    Grard Imbert (interface graphique, sprites, introduction, temps rel,
//                   mise en commun de tous les modules du programme)
//    Cyril Cohen (listes, algorithmes statistiques)
//    Johanna Nizard (temps rel, algorithmes statistiques)
//    Matthieu Fleurmont (algorithmes gnraux et debugging)

#include <stdio.h>
#include <allegro.h>

#include "sprite.h" //Routines de gestion des sprites.
#include "statb.h"  //Partie B du sujet.
#include "statc.h"  //Partie C du sujet.
#include "statd.h"  //Partie D du sujet.

int main(int arc, char *argv[])
{ int ret=0,frame,temp=0,i,j,temps,compteur=0,fin=0,pix32=0;
  int pixel,xpos,ypos,choix=0,vitesse=0;
  char c;
  BITMAP *ecran,*dessin, *logo, *gerintro, *cyrintro, *jointro, *matintro;
  BITMAP *titre, *titre2, *miage, *menu1, *menu2, *menu3, *menu4, *menu5;
  PALETTE pal;
  float parametre1=0.5,parametre2=0.8; //Paramtres par dfaut.
  sprite mario[10],curseur;

  //*****************************
  //* Routines d'initialisation *
  //*****************************

  clrscr();

  allegro_init();
  install_keyboard();
  install_mouse();

  //Il n'existe pas de "randomize" dans ce compilateur, juste une
  //fonction "srandom" qui intitialise le "seed" de la fonction
  //randomize selon une valeur passe en argument. Le problme
  //de la fonction srandom tant que pour une valeur fixe, les
  //valeurs pseudo-aleatoire seront toujours les mmes.
  //Pour rsoudre ce problme, on rcupre le temps du systme
  //et on le passe en argument. Ainsi on obtient des rsultats
  //un peu plus "alatoires" et surtout diffrents  chaque xcution.

  srandom(time(0));

  //Routines de dtection du mode graphique.
  //Si le mode graphique (640x480 en 16 millions de couleurs)
  //n'est pas disponible, on quitte le programme.

  //Le problme est que le standard VESA est implment diffrement
  //par chaque constructeur, il faut donc dterminer:
  // 1) Si le mode est disponible,
  // 2) Si ce mode est en 32 bits (4 octets par pixel) ou
  //    bien 24 bits (3 octets) par pixel  l'cran.
  // 3) Rcuperer la structure du pixel.
  // 4) Dfinir le mode de fonctionnement de la mmoire vido.

  //    Les modes 32 et 24 bits sont les mmes visuellement (16 millions de couleurs
  //    possibles pour chaque pixel), mais au niveau interne, le mode
  //    32 bits est extrmement plus rapide: il est plus ais 
  //    l'ordinateur de manipuler des mots de 4 octets (DWORD) qu'un
  //    mot de deux octets puis un octet suel (WORD puis BYTE).
  //    Cela se passe au niveau du processeur:
  //    En assembleur, le mode 32 bits serait utilis ainsi:
  //       mov eax,PIXEL  (on met la valeur du pixel dans EAX)
  //       mov ECRAN,eax  (puis on transmet cette valeur dans AX)
  //    Tandis qu'en mode 24 bits:
  //       mov eax,PIXEL  (on met la valeur du pixel dans EAX)
  //       mov ECRAN,ax   (on dplace le mot de 2 octets  l'cran)
  //       shr eax,16     (on dcale tout le registre pour rcuprer
  //                        le troisime octet)
  //       mov ECRAN,AL   (on dplace l'octet restant)
  //    Comme vous pouvez le constater, deux instructions supplmentaires
  //    font leur apparition, ce qui peut plus que doubler le temps
  //    de traitement. L'cran est redessin plusieurs fois par seconde,
  //    et le mode graphique contient 640x480 soit plus de 900000 pixels !
  //    D'o le gain de vitesse considrable en utilisant le mode 32 bits.
  //    En mode 24 bits, chaque octet du pixel (3 au total) reprsente
  //    une composante de couleur pouvant prendre une valeur allant de 0
  //     255. Ainsi 255^3 = 16 millions de couleurs. Le problme, est que
  //    le programme doit dtecter quel est l'ordre de ces composantes:
  //    celui ci peut tre RGB (rouge,vert,bleu) ou bien suivre l'organisation
  //    de la mmoire type Big Indian (c'est  dire tout est stock  l'envers)
  //    et donc BGR. Il existe mme des modes hybrides comme GBR ou BRG...
  //    Avec 32 bits, la chose est encore pire. Le quatrime octet est inutilis
  //    donc on a une structure de type XRGB (X=vide, d'ailleurs il est possible
  //    de stocker des informations dans ces octets vides, ce qui peut tre
  //    intressant pour certains effets graphiques ou optimisations).
  //    Il est donc ncessaire de connatre cette structure et d'organiser
  //    le restant des fonctions grce  cela.
  //    Heureusement tout ceci se fait au niveau du driver graphique, et
  //    donc le programme crit en C peut utiliser les fonctions graphiques
  //    de manire transparente.

  set_color_depth(32);
  ret=set_gfx_mode(GFX_AUTODETECT,640,480,0,0);
  if(ret!=0)
    { set_color_depth(24);
      ret=set_gfx_mode(GFX_AUTODETECT,640,480,0,0);
      if(ret!=0)
        { printf("Mode graphique 640*480 en 16 millions de couleurs non disponible.\n");
          printf("La carte graphique de votre ordinateur doit avoir au moins 1 Mo de\n");
          printf("mmoire vido. Si tel est le cas, essayez d'installer un driver VESA\n");
          printf("comme UniVBE ou Scitech Display Doctor.\n");
          exit(-1);
        }
    }

  //**********************
  //* Debut du programme *
  //**********************

  ecran=create_bitmap(640,480); //On cre le double buffer de l'cran.
  clear(ecran);

  //Dbut de la squence d'introduction

  gerintro=create_bitmap(35,98);//Cration des autres objets graphiques
  cyrintro=create_bitmap(33,84);
  jointro=create_bitmap(76,85);
  matintro=create_bitmap(38,95);
  titre=create_bitmap(111,62);
  titre2=create_bitmap(111,62);
  miage=create_bitmap(284,110);
  logo=create_bitmap(89,71);
  clear(logo);

  //chargement des images  partir de fichiers PCX
  //l'argument "pal" est utilis dans les mode 256 couleurs, donc
  //sa valeur n'a pas d'importance ici (c'est pourquoi la variable
  //a t cre mais n'est pas initialise).

  logo=load_pcx("finelogo.pcx", pal);
  gerintro=load_pcx("ger.pcx", pal);
  cyrintro=load_pcx("cyr.pcx", pal);
  jointro=load_pcx("jo.pcx", pal);
  matintro=load_pcx("mat.pcx",pal);
  titre=load_pcx("titre.pcx",pal);
  miage=load_pcx("miage.pcx",pal);
  clear(titre2);

  //Certains moniteurs mettent un petit temps avant d'initialiser
  //le mode graphique, mais le programme continue  tourner.
  //Afin de ne rien perdre de la squence d'introduction, on
  //attend que l'utilisateur voie un message et appuie sur une
  //touche pour commencer.

  textprintf(screen,font,190,220,243,"Appuyez sur ESPACE pour commencer.");

  while(!key[KEY_SPACE])
    {}

  clear_keybuf();

  //"Fade to blue"
  //On passe du noir vers un "bleu Dauphine" (du logo)
  for(i=0;i<256;i+=16)
    { pixel=makecol(i,i,i);
      clear_to_color(screen,i);
    }

  //Zoom arrire du logo de Dauphine.
  i=480;
  while(i>71)
   { stretch_sprite(ecran,logo,0,0,i*4/3,i);
     blit(ecran,screen,0,0,0,0,640,480);
     clear(ecran);
     if(i>270)
       i-=11;
     else
       i-=6;
   }
  draw_sprite(screen,logo,0,0);
  destroy_bitmap(logo);

  //On dtruit le premier logo et on charge un autre plus complet.

  logo=create_bitmap(384,80);
  clear(logo);
  logo=load_pcx("dauphine.pcx",pal);

  pixel=makecol(30,30,200);

  //Dessin du logo complet.

  for(i=89;i<385;i+=4)
   { masked_blit(logo,ecran,0,0,0,0,i,80);
     line(ecran,i+1,0,i+1,80,pixel);
     blit(ecran,screen,0,0,0,0,640,480);
     line(ecran,i+1,0,i+1,80,0);
   }

  blit(ecran,screen,0,0,0,0,640,480);

   for(i=378;i>0;i-=4)
    { line(ecran,i,80,378,80,pixel);
      blit(ecran,screen,0,0,0,0,640,480);
    }

  destroy_bitmap(logo);

  //Apparition des programmeurs en haut  droite

  for(i=639;i>399;i--)
    { draw_sprite(screen,gerintro,i+95,10);
      draw_sprite(screen,matintro,i+150,24);
      draw_sprite(screen,cyrintro,i+200,24);
      draw_sprite(screen,jointro,i,20);
    }

  //Zoom arrire du titre: "Stat Project".
  //(le "Stat Fighter" d'origine tait destin complter une
  //squence d'introduction plus longue mais abandone car
  //compltement superflue et lourde en espace disque)

  while(i>201)
  { stretch_sprite(screen,titre,350-i,140,i*4/3,i);
    stretch_sprite(screen,titre2,350-i,140,i*4/3,i);
    i-=6;
  }

  stretch_sprite(screen,titre,350-i,140,i*4/3,i);
    
  draw_sprite(screen,miage,350,365);

  //On a plus besoin de tous les objets graphiques alors on les dtruit.

  destroy_bitmap(titre);
  destroy_bitmap(titre2);
  destroy_bitmap(gerintro);
  destroy_bitmap(cyrintro);
  destroy_bitmap(jointro);
  destroy_bitmap(miage);
  destroy_bitmap(matintro);

  //Fin des effets d'introduction.

  //On sauvegarde tout l'cran dans le double buffer pour l'animation
  //des Marios (les bonhommes  casquette rouge).

  blit(screen,ecran,0,0,0,0,640,480);

  temps=0;

  //Initialisation des Marios:
  //On utilisera uniquement 10 Marios. Quand un Mario aura atteint un
  //bord de l'cran, on l'emmne instantanment  l'autre bord.

  for(i=0;i<10;i++)
   { chargerMario((sprite_ptr)&mario[i]);
     setpos((sprite_ptr)&mario[i],640+i*63,300);
     scan_sous_sprite((sprite_ptr)&mario[i],ecran);
   }

  //Boucle de fin d'introduction: on attend que l'utilisateur clique la
  //souris pour continuer.

  while(!(mouse_b&1))
    { compteur++;
      //Boucle de dplacement des Marios.
      for(i=0;i<10;i++)
       { effacer_sprite((sprite_ptr)&mario[i],ecran);
         setpos((sprite_ptr)&mario[i],mario[i].x-2,mario[i].y);
         if(compteur==4)
           { mario[i].curr_frame++;
           }
         if(mario[i].curr_frame>2)
           mario[i].curr_frame=0;
         if(mario[i].x<1)
            mario[i].x=640;
         scan_sous_sprite((sprite_ptr)&mario[i],ecran);
         dessiner_sprite((sprite_ptr)&mario[i],ecran);
       }
      if(compteur==4)
      compteur=0;

      //clignotement du texte:

      if(temps>25)
        { textout(screen,font,"CLICKEZ POUR COMMENCER",50,420,pixel);

         if(temps>50) temps=0;

        }
      else
         textout(screen,font,"CLICKEZ POUR COMMENCER",50,420,0);
          
      blit(ecran,screen,0,0,0,0,640,480);
      temps++;

    }

  //On vide le buffer clavier, si une touche a t appuye.
  clear_keybuf();

  //Fin de l'introduction

  //Dbut du Menu Principal

  //Chaque lment du menu qui est "anim" est en fait une
  //image contenant deux parties: une correspondant  l'image quand
  //l'option est "active" et l'autre quand elle ne l'est pas.
  //Ainsi on affiche l'une ou l'autre selon la position du curseur.

  menu1=create_bitmap(223,41);
  menu1=load_pcx("menu1.pcx",pal);

  menu2=create_bitmap(213,109);
  menu2=load_pcx("menu2.pcx",pal);

  menu3=create_bitmap(438,85);
  menu3=load_pcx("menu3.pcx",pal);

  menu4=create_bitmap(432,153);
  menu4=load_pcx("menu4.pcx",pal);

  menu5=create_bitmap(368,165);
  menu5=load_pcx("menu5.pcx",pal);

  //Chargement des images d'animation du curseur
  chargerCurseur((sprite_ptr)&curseur);

  //Boucle principale du programme:
  //on continue  tourner tant que l'utilisateur n'a pas choisi de quitter.
  while(!fin)
  { clear(screen);
    clear(ecran);

    frame=0;
    temps=0;
    choix=0;

  //On a dja initialis les Marios auparavant, pour l'introduction.
  //On se contente alors de modifier leur position.

  for(i=0;i<10;i++)
   { setpos((sprite_ptr)&mario[i],640+i*63,90);
     scan_sous_sprite((sprite_ptr)&mario[i],ecran);
   }
  j=0;
  i=0;

  scan_sous_sprite((sprite_ptr)&curseur,ecran);

  pixel=makecol(150,45,150);  //On cre un pixel  partir de trois composantes R,G,B.

  textout(ecran,font,"Projet de statistiques -- IUP1 MIAGE -- Universite Paris IX Dauphine",50,10,pixel);
  pixel=makecol(100,45,150);
  textout(ecran,font," Gerard Imbert - Cyril Cohen - Johanna Nizard - Matthieu Fleurmont",50,20,pixel);
  blit(menu1,ecran,0,0,200,40,223,41);

  //A prsent on fait marcher les Marios et on dplace le curseur de
  //la souris tant que l'utilisateur n'a pas cliqu.

  while(!(mouse_b&1))
  { effacer_sprite((sprite_ptr)&curseur,ecran);

    xpos=mouse_x;   //On rcupre la position actuelle de la souris.
    ypos=mouse_y;

    setpos((sprite_ptr)&curseur,xpos,ypos);

    //Activation/Dsactivation des lments du menu:

    //Option "QUITTER"
    if((xpos>200)&&(xpos<411)&&(ypos>400)&&(ypos<453))
      { blit(menu2,ecran,1,1,200,400,211,53);
        choix=2;
      }
    else
      blit(menu2,ecran,1,55,200,400,211,53);

    //Option "ETUDE DES ARRIVEES"
    if((xpos>95)&&(xpos<533)&&(ypos>140)&&(ypos<181))
      { blit(menu3,ecran,1,1,95,140,436,41);
        choix=3;
      }
    else
      blit(menu3,ecran,1,43,95,140,436,41);

    //Option de la partie C
    if((xpos>100)&&(xpos<530)&&(ypos>200)&&(ypos<275))
      { blit(menu4,ecran,1,1,100,200,430,75);
        choix=4;
      }
    else
      blit(menu4,ecran,1,77,100,200,430,75);

    //Option "REPRESENTATION GRAPHIQUE"
    if((xpos>130)&&(xpos<496)&&(ypos>300)&&(ypos<381))
      { blit(menu5,ecran,1,1,130,300,366,81);
        choix=5;
      }
    else
      blit(menu5,ecran,1,83,130,300,366,81);

    //Options de paramtrage:
    i=makecol(100,100,244);

    textprintf(ecran,font,0,390,i,"Parametre");
    textprintf(ecran,font,0,400,i,"generant");
    textprintf(ecran,font,0,410,i,"les arrivees: %.2f",parametre1);
    if((xpos>100)&&(xpos<150)&&(ypos>390)&&(ypos<410))
      { textprintf(ecran,font,120,400,i*50,"+");
        choix=6;
      }
    else
      textprintf(ecran,font,120,400,i,"+");
    if((xpos>100)&&(xpos<150)&&(ypos>410)&&(ypos<430))
      { textprintf(ecran,font,120,420,i*50,"-");
        choix=7;
      }
    else
      textprintf(ecran,font,120,420,i,"-");

    textprintf(ecran,font,500,380,i,"        Parametre");
    textprintf(ecran,font,500,390,i,"     generant les");
    textprintf(ecran,font,500,400,i,"         temps de");
    textprintf(ecran,font,500,410,i,"%.2f : traitement",parametre2);
    if((xpos>490)&&(xpos<540)&&(ypos>390)&&(ypos<410))
      { textprintf(ecran,font,510,400,i*50,"+");
        choix=8;
      }
    else
      textprintf(ecran,font,510,400,i,"+");
    if((xpos>490)&&(xpos<540)&&(ypos>410)&&(ypos<430))
      { textprintf(ecran,font,510,420,i*50,"-");
        choix=9;
      }
    else
      textprintf(ecran,font,510,420,i,"-");

    //Dplacement des Marios:

    setframe((sprite_ptr)&curseur,j);
    if(temps>2)
    { if(frame==0)
        j++;
      else
        j--;
      if(j==3)
        frame=1;
      if(j==0)
        frame=0;
      temps=0;
    }
    temps++;

    compteur++;
    for(i=0;i<10;i++)
       { effacer_sprite((sprite_ptr)&mario[i],ecran);
         setpos((sprite_ptr)&mario[i],mario[i].x-2,mario[i].y);
         if(compteur==4)
           { mario[i].curr_frame++;
           }
         if(mario[i].curr_frame>2)
           mario[i].curr_frame=0;
         if(mario[i].x<1)
            mario[i].x=640;
       }


    scan_sous_sprite((sprite_ptr)&curseur,ecran);
    for(i=0;i<10;i++)
       scan_sous_sprite((sprite_ptr)&mario[i],ecran);

    for(i=0;i<10;i++)
       dessiner_sprite((sprite_ptr)&mario[i],ecran);

    dessiner_sprite((sprite_ptr)&curseur,ecran);

    if(compteur==4)
      compteur=0;

    //Mise  jour de l'cran  partir du Double Buffer:

    blit(ecran,screen,0,0,0,0,640,480);
  }//Fin while(NON CLICK)

  //Selon le choix de l'utilisateur on va  la partie choisie.
  //On utilise des "if" plutt qu'un "switch case" car
  //il y a peu d'instructions et un "switch case" impliquerait
  //l'ajout de plusieurs "break", augmentant sensiblement
  //le nombre d'instructions.

  if(choix==3)
  //Lancement de la partie B
    { choix==0;
      B_arrivees(parametre1);
    }
  if(choix==4)
  //Lancement de la partie C
    { choix==0;
      C_file(parametre1,parametre2);
    }
  if(choix==5)
  //Lancement de la partie D
    { choix==0;
      D_gfx_journee(parametre1,parametre2,&i,0);
    }
  if(choix==6)
  //Augmentation du paramtre gnrant les arrives
    { choix==0;
      parametre1+=0.01;
      if(parametre1>0.99)
        parametre1=0.99;
     }
  if(choix==7)
  //Diminution du paramtre gnrant les arrives
    { choix==0;
      parametre1-=0.01;
      if(parametre1<0.01)
        parametre1=0.01;
    }

  if(choix==8)
  //Augmentation du paramtre gnrant les temps de service
    { choix==0;
      parametre2+=0.01;
      if(parametre2>0.99)
        parametre2=0.99;
     }
  if(choix==9)
  //Diminution du paramtre gnrant les temps de service
    { choix==0;
      parametre2-=0.01;
      if(parametre2<0.01)
        parametre2=0.01;
    }
  if(choix==10)
  //Changement de la vitesse pour la partie D. Cette option n'a pas t
  //garde pour la version finale mais a t utile pour tester.
  //La vitesse lente correspond  des units de temps de 0.01 minutes
  //tandis que la vitesse rapide utilise des units de temps de 0.1 minutes.
    { vitesse=!vitesse;
    }
  if(choix==2)
  //FIN DU PROGRAMME
     fin=1;

  }

  //On libre la mmoire prise par les Marios,le curseur, et les images.

  for(i=0;i<10;i++)
    { sprite_detruit((sprite_ptr)&mario[i]);
    }
  sprite_detruit((sprite_ptr)&curseur);
  destroy_bitmap(menu1);
  destroy_bitmap(menu2);
  destroy_bitmap(menu3);
  destroy_bitmap(menu4);
  destroy_bitmap(menu5);

  clear_keybuf();

  //***********************
  //* Sortie du programme *
  //***********************
  //On revient au mode texte
  set_gfx_mode(GFX_TEXT,0,0,0,0);
  clrscr();

  printf("Projet de statistiques - IUP1 MIAGE - Universit Paris IX Dauphine - Mai 1998\n");
  printf("-----------------------------------------------------------------------------\n\n");
  printf("Interface graphique        : Grard Imbert        \"il est dj 3h du matin ?\"\n\n");
  printf("Structuration du programme : Cyril Cohen               \"j'ai perdu mes notes\"\n");
  printf("                             Johanna Nizard \"va falloir programmer une bombe\"\n");
  printf("\t\t\t     Grard Imbert            \"ne vous en faites pas\"\n\n");
  printf("Algorythmes et debugging   : Matthieu Fleurmont    \"je veux mes 250 feuilles\"\n");
  printf("                             Johanna Nizard                             \"euh\"\n");
  printf("                             Cyril Cohen                 \"on est  la bourre\"\n\n");
  printf("Textes et dessins          : Grard Imbert\"Au secours je n'ai plus d'ides !\"\n");
  printf("\n\n");
  printf("\t   Compil avec DJGPP v2.0 et Allegro v3.0 en Mai 1998\n\n");
  printf("\n\n\n\n");

  return 0;
}
