//Projet de statistiques - IUP1 MIAGE
//PRIMITIVES DE GESTION DE SPRITES
//Par Grard Imbert

#include "sprite.h"

//Sprite_init initialise une structure de sprite.
//sprite: le pointeur sur la structure
//x: la coordonne horizontale d'origine
//y: la coordonne verticale d'origine
//hauteur: la hauteur en pixels du sprite
//largeur: la largeur en pixels du sprite
//c1,c2,c3: la valeur initiale de divers compteurs.
void sprite_init(sprite_ptr sprite,int x, int y, int hauteur, int largeur, int c1, int c2, int c3)
{ int i;
  sprite->x=x;
  sprite->y=y;
  sprite->hauteur=hauteur;
  sprite->largeur=largeur;
  sprite->state=SPRITE_MORT;
  sprite->curr_frame=0;
  sprite->compteur_1=c1;
  sprite->compteur_2=c2;
  sprite->compteur_3=c3;
  sprite->num_frames=0;
  sprite->background=create_bitmap(largeur,hauteur);
  for(i=0;i<MAX_SPRITE_FRAMES;i++)
    { sprite->frames[i]=NULL;
    }
  return;
}

//sprite_detruit libre la mmoire utilise par un sprite.
void sprite_detruit(sprite_ptr sprite)
{ int i;
  destroy_bitmap(sprite->background);
  for(i=0;i<MAX_SPRITE_FRAMES;i++)
    { destroy_bitmap(sprite->frames[i]);
    }
  return;
}

//prendre_sprite capture les images composant les squences d'animation
//d'un sprite (frames en anglais)  partir d'une image PCX.
//image: l'image PCX charge
//sprite: le pointeur sur le sprite
//sprite_frame: le numro de frame o stocker l'image capture
//cell_x,cell_y: les coordonnes de la cellule  prendre
//L'image est compose de cellules ayant la mme taille que le sprite,
//et separes par une ligne de un pixel de largeur. Ainsi si
//cell_x=2 et cell_y=0 on prendra la troisime image de la premire
//range de l'image.
void prendre_sprite(BITMAP *image, sprite_ptr sprite, int sprite_frame, int cell_x, int cell_y)
{ int x_off,y_off,y,largeur,hauteur;
  BITMAP *sprite_data;

  sprite->num_frames++;

  largeur=sprite->largeur;
  hauteur=sprite->hauteur;
  sprite->frames[sprite_frame]=create_bitmap(largeur,hauteur);
  sprite_data=sprite->frames[sprite_frame];

  x_off=(largeur+1)*cell_x+1;
  y_off=(hauteur+1)*cell_y+1;

  blit(image,sprite_data,x_off,y_off,0,0,largeur,hauteur);

  return;
}

//sprite_collision teste si deux sprites se chevauchent.
int sprite_collision(sprite_ptr curseur, sprite_ptr menu)
{ if(((curseur->x>=menu->x)&&(curseur->x<menu->x+menu->largeur)) || ((curseur->x+curseur->largeur>=menu->x)&&(curseur->x+curseur->largeur<menu->x+menu->largeur) && (curseur->y>=menu->y)&&(curseur->y<menu->y+menu->hauteur))||((curseur->y+curseur->hauteur>=menu->y)&&(curseur->y+curseur->hauteur<menu->y+menu->hauteur)))
    return 1;
  return 0;
}

//chargerMario fait tout le ncessaire pour charger un sprite "Mario"
// partir d'une image PCX.
void chargerMario(sprite_ptr sprite)
{ int frame;
  PALETTE pal;
  BITMAP *dessin;
  dessin=create_bitmap(32,16);
  dessin=load_pcx("mario.pcx",pal);
  sprite_init(sprite,0,0,32,16,0,0,0);
  for(frame=0;frame<5;frame++)
    prendre_sprite(dessin,sprite,frame,frame,0);
  destroy_bitmap(dessin);
  return;
}

//chargerCurseur remplit la mme fonction que chargerMario mais pour
//le sprite du curseur anim.
void chargerCurseur(sprite_ptr sprite)
{ int frame;
  PALETTE pal;
  BITMAP *dessin;
  dessin=create_bitmap(132,34);
  dessin=load_pcx("cursor.pcx",pal);
  sprite_init(sprite,0,0,32,32,0,0,0);
  for(frame=0;frame<4;frame++)
    prendre_sprite(dessin,sprite,frame,frame,0);
  destroy_bitmap(dessin);
  return;
}

//dessiner_sprite dessine le sprite sur l'objet graphique de
//destination. Cet objet graphique peut tre l'cran, le Double
//Buffer, ou une autre image. On dessine automatiquement
//la frame courante et  la position du sprite courante.
void dessiner_sprite(sprite_ptr sprite,BITMAP *destination)
{ int x,y,largeur,hauteur;
  BITMAP *sprite_data;
  sprite_data=sprite->frames[sprite->curr_frame];
  x=sprite->x;
  y=sprite->y;
  hauteur=sprite->hauteur;
  largeur=sprite->largeur;
  masked_blit(sprite_data,destination,0,0,x,y,largeur,hauteur);
  return;
}

//scan_sous_sprite sauvegarde la zone rectangulaire situe en
//dessous du sprite. Ainsi on peut la restaurer quand le sprite
//est effac et il n'est pas ncessaire de redessiner tout
//l'cran (ou l'objet graphique de destination).
void scan_sous_sprite(sprite_ptr sprite, BITMAP *destination)
{ BITMAP *back_buffer;
  int x,y,hauteur,largeur;
  back_buffer=sprite->background;
  x=sprite->x;
  y=sprite->y;
  hauteur=sprite->hauteur;
  largeur=sprite->largeur;
  blit(destination,back_buffer,x,y,0,0,largeur,hauteur);
  return;
}

//effacer_sprite efface le sprite en dessinant par dessus
//de ce dernier l'image qu'on avait prcedemment capture
//via scan_sous_sprite.
void effacer_sprite(sprite_ptr sprite, BITMAP *destination)
{ BITMAP *back_buffer;
  int x,y,hauteur,largeur;
  back_buffer=sprite->background;
  x=sprite->x;
  y=sprite->y;
  hauteur=sprite->hauteur;
  largeur=sprite->largeur;
  blit(back_buffer,destination,0,0,x,y,largeur,hauteur);
  return;
}

//setframe slectionne la frame courante du sprite.
void setframe(sprite_ptr sprite, int frame)
{ sprite->curr_frame=frame;
  return;
}

//setpos modifie les coordonnes du sprite.
void setpos(sprite_ptr sprite, int x, int y)
{ sprite->x=x;
  sprite->y=y;
  return;
}

